@@ -5,8 +5,8 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2009, Robert van Engelen, Genivia Inc. All Rights Reserved.
-This software is released under one of the following two licenses:
+Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
 GPL license.
@@ -49,6 +49,7 @@ class Types
     MapOfStringToString deftypemap;
     MapOfStringToString usetypemap;
     MapOfStringToString ptrtypemap;
+    MapOfStringToString smptypemap;
     MapOfPairToString	qnames;	// (URI,name) -> name
     MapOfStringToString	uris;	// URI -> prefix
     MapOfStringToNum	syms;	// prefix -> count (ns1, ns2, ...)
@@ -68,42 +69,47 @@ class Types
     const char *fname(const char *prefix, const char *URI, const char *qname, SetOfString *reserved, enum Lookup lookup, bool isqname);
   public:
     const char *aname(const char *prefix, const char *URI, const char *qname);
+    const char *wname(const char *prefix, const char *URI, const char *qname);
     const char *cname(const char *prefix, const char *URI, const char *qname);
     const char *tname(const char *prefix, const char *URI, const char *qname);
     const char *tnameptr(bool, const char *prefix, const char *URI, const char *qname);
-    const char *pname(bool flag, const char *prefix, const char *URI, const char *qname);
+    const char *tnamenoptr(const char *prefix, const char *URI, const char *qname);
+    const char *pname(bool flag, bool smart, const char *prefix, const char *URI, const char *qname);
     const char *oname(const char *prefix, const char *URI, const char *qname);
     const char *ename(const char *type, const char *value, bool isqname);
     const char *sname(const char *URI, const char *name);
     const char *gname(const char *URI, const char *name);
     const char *uname(const char *URI);
+    const char *vname(const char *var);
     const char *nsprefix(const char *prefix, const char *URI);
-    const char *deftname(enum Type type, const char *pointer, bool is_pointer, const char *prefix, const char *URI, const char *qname);
+    const char *prefix(const char *name);
+    const char *uri(const char *name);
+    const char *deftname(enum Type type, bool mk_pointer, bool is_pointer, const char *prefix, const char *URI, const char *qname);
     bool is_defined(const char *prefix, const char *URI, const char *qname);
     bool is_nillable(const xs__element& element);
-    bool is_basetypeforunion(const char *prefix, const char *URI, const char *type);
-    bool is_basetype(const char *prefix, const char *URI, const char *type);
+    bool is_choicetype(const char *prefix, const char *URI, const char *type);
+    bool is_ptr(const char *prefix, const char *URI, const char *type);
     void dump(FILE*);
     void define(const char *URI, const char *name, const xs__complexType&);
     void gen(const char *URI, const vector<xs__attribute>&);
     void gen(const char *URI, const vector<xs__attributeGroup>&);
     void gen(const char *URI, const vector<xs__all>&);
-    void gen(const char *URI, const vector<xs__element>&);
+    void gen(const char *URI, const vector<xs__element>&, const char *minOccurs, const char *maxOccurs);
     void gen(const char *URI, const vector<xs__group>&);
     void gen(const char *URI, const vector<xs__any>&);
     void gen(const char *URI, const vector<xs__contents>&);
-    void gen(const char *URI, const char *name, const xs__simpleType&, bool anonymous);
+    void gen(const char *URI, const char *name, const xs__simpleType&, bool anonymous, bool nested_restriction);
     void gen(const char *URI, const char *name, const xs__complexType&, bool anonymous);
     void gen(const char *URI, const xs__attribute&);
-    void gen(const char *URI, const xs__all&);
-    void gen(const char *URI, const xs__seqchoice&);
-    void gen(const char *URI, const char *name, const xs__seqchoice&);
-    void gen(const char *URI, const xs__element&, bool substok);
-    void gen(const char *URI, const xs__group&);
-    void gen(const char *URI, const xs__any&);
+    void gen(const char *URI, const xs__all&, const char *minOccurs, const char *maxOccurs);
+    void gen(const char *URI, const xs__seqchoice&, const char *minOccurs, const char *maxOccurs);
+    void gen(const char *URI, const char *name, const xs__seqchoice&, const char *minOccurs, const char *maxOccurs);
+    void gen(const char *URI, const xs__element&, bool substok, const char *minOccurs, const char *maxOccurs);
+    void gen(const char *URI, const xs__group&, const char *minOccurs, const char *maxOccurs);
+    void gen(const char *URI, const xs__any&, const char *minOccurs, const char *maxOccurs);
     void gen(const char *URI, const xs__anyAttribute&);
     void gen_inh(const char *URI, const xs__complexType *complexType, bool anonymous);
-    void gen_soap_array(const char *name, const char *t, const char *item, const char *type);
+    void gen_soap_array(const char *t, const char *item, const char *type);
     void gen_substitutions(const char *URI, const xs__element &element);
     void document(const xs__annotation*);
     void modify(const char *name);
