@@ -5,7 +5,7 @@
 	Supports both Basic and Digest authentication.
 
 gSOAP XML Web services tools
-Copyright (C) 2000-2013, Robert van Engelen, Genivia Inc., All Rights Reserved.
+Copyright (C) 2000-2015, Robert van Engelen, Genivia Inc., All Rights Reserved.
 This part of the software is released under one of the following licenses:
 GPL, the gSOAP public license, or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -20,7 +20,7 @@ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the License.
 
 The Initial Developer of the Original Code is Robert A. van Engelen.
-Copyright (C) 2000-2013, Robert van Engelen, Genivia, Inc., All Rights Reserved.
+Copyright (C) 2000-2015, Robert van Engelen, Genivia, Inc., All Rights Reserved.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -57,10 +57,15 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 /**
 
-@page httpda The http_da plugin for clients and stand-alone services
+@page httpda The HTTP-DA plugin
+
+[TOC]
+
+@section httpda_0 HTTP-DA Setup
 
 Additional build steps required:
-- Compile all sources with -DWITH_OPENSSL
+
+- Compile all sources with `-DWITH_OPENSSL`
 - Link libgsoapssl (libgsoapssl++), or use the lib's stdsoap2.c/.cpp source
 - Compile and link with plugin/httpda.c, plugin/md5evp.c, and plugin/threads.c
 
@@ -70,10 +75,10 @@ HTTP Basic Authentication is the default authentication supported by gSOAP. The
 credentials for client-side use age set with:
 
 @code
-soap.userid = "<userid>";
-soap.passed = "<passwd>";
-if (soap_call_ns__method(&soap, ...))
-  ... // error
+    soap.userid = "<userid>";
+    soap.passed = "<passwd>";
+    if (soap_call_ns__method(&soap, ...))
+      ... // error
 @endcode
 
 HTTP Basic Authentication should never be used over plain HTTP, because the
@@ -87,113 +92,113 @@ exchange.
 To use HTTP Digest Authentication with gSOAP, register the http_da plugin:
 
 @code
-#include "httpda.h"
-soap_register_plugin(&soap, http_da);
+    #include "httpda.h"
+    soap_register_plugin(&soap, http_da);
 @endcode
 
 To make a client-side service call:
 
 @code
-struct http_da_info info;
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 401)
-  {
-    http_da_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
-    if (soap_call_ns__method(&soap, ...)) // try again
-      ... // error
-    http_da_release(&soap, &info);
-  }
-  else
-    ... // other error
-}
+    struct http_da_info info;
+    if (soap_call_ns__method(&soap, ...))
+    {
+      if (soap.error == 401)
+      {
+	http_da_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
+	if (soap_call_ns__method(&soap, ...)) // try again
+	  ... // error
+	http_da_release(&soap, &info);
+      }
+      else
+	... // other error
+    }
 @endcode
 
 The "<authrealm>" is a string that is associated with the server's realm. It
 can be obtained after an unsuccessful non-authenticated call:
 
 @code
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 401) // HTTP authentication is required
-  {
-    const char *realm = soap.authrealm;
-    ...
-  }
-  else
-    ... // error
-}
+    if (soap_call_ns__method(&soap, ...))
+    {
+      if (soap.error == 401) // HTTP authentication is required
+      {
+	const char *realm = soap.authrealm;
+	...
+      }
+      else
+	... // error
+    }
 @endcode
 
 Before a second call is made to the same endpoint that requires authentication,
 you must restore the authentication state and then finally release it:
 
 @code
-struct http_da_info info;
-bool auth = false;
+    struct http_da_info info;
+    bool auth = false;
 
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 401)
-  {
-    http_da_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
-    auth = true;
-  }
-  else
-    ... // other error
-}
+    if (soap_call_ns__method(&soap, ...))
+    {
+      if (soap.error == 401)
+      {
+	http_da_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
+	auth = true;
+      }
+      else
+	... // other error
+    }
 
-if (soap_call_ns__method(&soap, ...))
-  ... // error
+    if (soap_call_ns__method(&soap, ...))
+      ... // error
 
-if (auth)
-  http_da_restore(&soap, &info);
-if (soap_call_ns__method(&soap, ...))
-  ... // error
+    if (auth)
+      http_da_restore(&soap, &info);
+    if (soap_call_ns__method(&soap, ...))
+      ... // error
 
-soap_destroy(&soap); // okay to dealloc data
-soap_end(&soap);     // okay to dealloc data
+    soap_destroy(&soap); // okay to dealloc data
+    soap_end(&soap);     // okay to dealloc data
 
-if (auth)
-  http_da_restore(&soap, &info);
-if (soap_call_ns__method(&soap, ...))
-  ... // error
+    if (auth)
+      http_da_restore(&soap, &info);
+    if (soap_call_ns__method(&soap, ...))
+      ... // error
 
-if (auth)
-  http_da_release(&soap, &info);
+    if (auth)
+      http_da_release(&soap, &info);
 
-soap_destroy(&soap);
-soap_end(&soap);
-soap_done(&soap);
+    soap_destroy(&soap);
+    soap_end(&soap);
+    soap_done(&soap);
 @endcode
 
 For HTTP proxies requiring HTTP Digest Authenticaiton, use the 'proxy'
 functions:
 
 @code
-struct http_da_info info;
-...
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 407)
-  {
-    http_da_proxy_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
-    auth = true;
-  }
-  else
-    ... // error
-}
+    struct http_da_info info;
+    ...
+    if (soap_call_ns__method(&soap, ...))
+    {
+      if (soap.error == 407)
+      {
+	http_da_proxy_save(&soap, &info, "<authrealm>", "<userid>", "<passwd>");
+	auth = true;
+      }
+      else
+	... // error
+    }
 
-if (auth)
-  http_da_proxy_restore(&soap, &info);
-if (soap_call_ns__method(&soap, ...))
-  ... // error
+    if (auth)
+      http_da_proxy_restore(&soap, &info);
+    if (soap_call_ns__method(&soap, ...))
+      ... // error
 
-http_da_proxy_release(&soap, &info);
+    http_da_proxy_release(&soap, &info);
 
-soap_destroy(&soap);
-soap_end(&soap);
-soap_done(&soap);
+    soap_destroy(&soap);
+    soap_end(&soap);
+    soap_done(&soap);
 @endcode
 
 @section httpda_2 Client Example
@@ -201,51 +206,51 @@ soap_done(&soap);
 A client authenticating against a server:
 
 @code
-soap_register_plugin(&soap, http_da);
-// try calling without authenticating
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 401) // HTTP authentication is required
-  {
-    if (!strcmp(soap.authrealm, authrealm)) // check authentication realm
+    soap_register_plugin(&soap, http_da);
+    // try calling without authenticating
+    if (soap_call_ns__method(&soap, ...))
     {
-      struct http_da_info info; // to store userid and passwd
-      http_da_save(&soap, &info, authrealm, userid, passwd);
-      // call again, now with credentials
-      if (soap_call_ns__method(&soap, ...) == SOAP_OK)
+      if (soap.error == 401) // HTTP authentication is required
       {
-        ... // process response data
-        soap_end(&soap);
-	... // userid and passwd were deallocated (!)
-        http_da_restore(&soap, &info); // get userid and passwd after soap_end()
-        if (!soap_call_ns__method(&soap, ...) == SOAP_OK)
-	  ... // error
-        http_da_release(&soap, &info); // free data and remove userid and passwd
+	if (!strcmp(soap.authrealm, authrealm)) // check authentication realm
+	{
+	  struct http_da_info info; // to store userid and passwd
+	  http_da_save(&soap, &info, authrealm, userid, passwd);
+	  // call again, now with credentials
+	  if (soap_call_ns__method(&soap, ...) == SOAP_OK)
+	  {
+	    ... // process response data
+	    soap_end(&soap);
+	    ... // userid and passwd were deallocated (!)
+	    http_da_restore(&soap, &info); // get userid and passwd after soap_end()
+	    if (!soap_call_ns__method(&soap, ...) == SOAP_OK)
+	      ... // error
+	    http_da_release(&soap, &info); // free data and remove userid and passwd
 @endcode
 
 A client authenticating against a proxy:
 
 @code
-soap_register_plugin(&soap, http_da);
-// try calling without authenticating
-if (soap_call_ns__method(&soap, ...))
-{
-  if (soap.error == 407) // HTTP authentication is required
-  {
-    if (!strcmp(soap.authrealm, authrealm)) // check authentication realm
+    soap_register_plugin(&soap, http_da);
+    // try calling without authenticating
+    if (soap_call_ns__method(&soap, ...))
     {
-      struct http_da_info info; // to store userid and passwd
-      http_da_proxy_save(&soap, &info, authrealm, userid, passwd);
-      // call again, now with credentials
-      if (soap_call_ns__method(&soap, ...) == SOAP_OK)
+      if (soap.error == 407) // HTTP authentication is required
       {
-        ... // process response data
-        soap_end(&soap);
-	... // userid and passwd were deallocated (!)
-        http_da_proxy_restore(&soap, &info); // get userid and passwd after soap_end()
-        if (!soap_call_ns__method(&soap, ...) == SOAP_OK)
-	  ... // error
-        http_da_proxy_release(&soap, &info); // free data and remove userid and passwd
+	if (!strcmp(soap.authrealm, authrealm)) // check authentication realm
+	{
+	  struct http_da_info info; // to store userid and passwd
+	  http_da_proxy_save(&soap, &info, authrealm, userid, passwd);
+	  // call again, now with credentials
+	  if (soap_call_ns__method(&soap, ...) == SOAP_OK)
+	  {
+	    ... // process response data
+	    soap_end(&soap);
+	    ... // userid and passwd were deallocated (!)
+	    http_da_proxy_restore(&soap, &info); // get userid and passwd after soap_end()
+	    if (!soap_call_ns__method(&soap, ...) == SOAP_OK)
+	      ... // error
+	    http_da_proxy_release(&soap, &info); // free data and remove userid and passwd
 @endcode
 
 @section httpda_3 Server-Side Usage
@@ -253,79 +258,79 @@ if (soap_call_ns__method(&soap, ...))
 Server-side HTTP Basic Authentication can be enforced by simply checking the soap.userid and soap.passwd values in a service method that requires client authentication:
 
 @code
-soap_register_plugin(&soap, http_da);
-...
-soap_serve(&soap);
-...
-int ns__method(struct soap *soap, ...)
-{
-  if (!soap->userid || !soap->passwd || strcmp(soap->userid, "<userid>") || strcmp(soap->passwd, "<passwd>"))
-    return 401; // HTTP authentication required
-  ...
-}
+    soap_register_plugin(&soap, http_da);
+    ...
+    soap_serve(&soap);
+    ...
+    int ns__method(struct soap *soap, ...)
+    {
+      if (!soap->userid || !soap->passwd || strcmp(soap->userid, "<userid>") || strcmp(soap->passwd, "<passwd>"))
+	return 401; // HTTP authentication required
+      ...
+    }
 @endcode
 
 HTTP Digest Authentication is verified differently:
 
 @code
-soap_register_plugin(&soap, http_da);
-...
-soap_serve(&soap);
-...
-int ns__method(struct soap *soap, ...)
-{
-  if (soap->authrealm && soap->userid)
-  {
-    passwd = ... // database lookup on userid and authrealm to find passwd
-    if (!strcmp(soap->authrealm, authrealm) && !strcmp(soap->userid, userid))
-    { 
-      if (!http_da_verify_post(soap, passwd)) // HTTP POST DA verification
+    soap_register_plugin(&soap, http_da);
+    ...
+    soap_serve(&soap);
+    ...
+    int ns__method(struct soap *soap, ...)
+    {
+      if (soap->authrealm && soap->userid)
       {
-        ... // process request and produce response
-        return SOAP_OK;
+	passwd = ... // database lookup on userid and authrealm to find passwd
+	if (!strcmp(soap->authrealm, authrealm) && !strcmp(soap->userid, userid))
+	{ 
+	  if (!http_da_verify_post(soap, passwd)) // HTTP POST DA verification
+	  {
+	    ... // process request and produce response
+	    return SOAP_OK;
+	  }
+	}
       }
-    }
-  }
-  soap->authrealm = authrealm; // realm to send to client
-  return 401; // Not authorized, challenge with digest authentication
+      soap->authrealm = authrealm; // realm to send to client
+      return 401; // Not authorized, challenge with digest authentication
 @endcode
 
-The http_da_verify_post() function checks the HTTP POST credentials. To verify
-an HTTP GET operation, use http_da_verify_get().
+The `http_da_verify_post` function checks the HTTP POST credentials. To verify
+an HTTP GET operation, use `http_da_verify_get`.
 
 @section httpda_4 Server Example
 
 @code
-soap_register_plugin(&soap, http_da);
-...
-soap_serve(&soap);
-...
-int ns__method(struct soap *soap, ...)
-{
-  if (soap->userid && soap->passwd) // Basic authentication
-  {
-    if (!strcmp(soap->userid, userid) && !strcmp(soap->passwd, passwd))
+    soap_register_plugin(&soap, http_da);
+    ...
+    soap_serve(&soap);
+    ...
+    int ns__method(struct soap *soap, ...)
     {
-      ... // can also check soap->authrealm 
-      ... // process request and produce response
-      return SOAP_OK;
-    }
-  }
-  else if (soap->authrealm && soap->userid) // Digest authentication
-  {
-    passwd = ... // database lookup on userid and authrealm to find passwd
-    if (!strcmp(soap->authrealm, authrealm) && !strcmp(soap->userid, userid))
-    { 
-      if (!http_da_verify_post(soap, passwd)) // HTTP POST DA verification
+      if (soap->userid && soap->passwd) // Basic authentication
       {
-        ... // process request and produce response
-        return SOAP_OK;
+	if (!strcmp(soap->userid, userid) && !strcmp(soap->passwd, passwd))
+	{
+	  ... // can also check soap->authrealm 
+	  ... // process request and produce response
+	  return SOAP_OK;
+	}
       }
+      else if (soap->authrealm && soap->userid) // Digest authentication
+      {
+	passwd = ... // database lookup on userid and authrealm to find passwd
+	if (!strcmp(soap->authrealm, authrealm) && !strcmp(soap->userid, userid))
+	{ 
+	  if (!http_da_verify_post(soap, passwd)) // HTTP POST DA verification
+	  {
+	    ... // process request and produce response
+	    return SOAP_OK;
+	  }
+	}
+      }
+      soap->authrealm = authrealm; // realm to send to client
+      return 401; // Not authorized, challenge with digest authentication
     }
-  }
-  soap->authrealm = authrealm; // realm to send to client
-  return 401; // Not authorized, challenge with digest authentication
-}
 @endcode
 
 @section httpda_5 HTTP Digest Authentication Limitations
@@ -351,8 +356,8 @@ static struct http_da_session *http_da_session = NULL;
 /** HTTP DA session database lock */
 static MUTEX_TYPE http_da_session_lock = MUTEX_INITIALIZER;
 
-#define HTTP_DA_NONCELEN 21
-#define HTTP_DA_OPAQUELEN 9
+#define HTTP_DA_NONCELEN (21)
+#define HTTP_DA_OPAQUELEN (9)
 
 /******************************************************************************\
  *
@@ -381,7 +386,7 @@ static void http_da_session_cleanup();
 void http_da_calc_nonce(struct soap *soap, char nonce[HTTP_DA_NONCELEN]);
 void http_da_calc_opaque(struct soap *soap, char opaque[HTTP_DA_OPAQUELEN]);
 static void http_da_calc_HA1(struct soap *soap, void **context, const char *alg, const char *userid, const char *realm, const char *passwd, const char *nonce, const char *cnonce, char HA1hex[33]);
-static void http_da_calc_response(struct soap *soap, void **context, char HA1hex[33], const char *nonce, const char *ncount, const char *cnonce, const char *qop, const char *method, const char *uri, char entityHAhex[33], char response[33]);
+static void http_da_calc_response(struct soap *soap, void **context, char HA1hex[33], const char *nonce, const char *ncount, const char *cnonce, const char *qop, const char *method, const char *uri, char entityHAhex[33], char response[33], char responseHA[16]);
 
 /******************************************************************************\
  *
@@ -391,6 +396,7 @@ static void http_da_calc_response(struct soap *soap, void **context, char HA1hex
 
 int http_da(struct soap *soap, struct soap_plugin *p, void *arg)
 {
+  (void)arg;
   p->id = http_da_id;
   p->data = (void*)SOAP_MALLOC(soap, sizeof(struct http_da_data));
   p->fcopy = http_da_copy;
@@ -419,7 +425,7 @@ static int http_da_init(struct soap *soap, struct http_da_data *data)
   data->fprepareinitrecv = soap->fprepareinitrecv;
   soap->fprepareinitrecv = http_da_prepareinitrecv;
   data->context = NULL;
-  memset(data->digest, 0, sizeof(data->digest));
+  memset((void*)data->digest, 0, sizeof(data->digest));
   data->nonce = NULL;
   data->opaque = NULL;
   data->qop = NULL;
@@ -427,7 +433,7 @@ static int http_da_init(struct soap *soap, struct http_da_data *data)
   data->nc = 0;
   data->ncount = NULL;
   data->cnonce = NULL;
-  data->response = NULL;
+  memset((void*)data->response, 0, sizeof(data->response));
 
   return SOAP_OK;
 }
@@ -435,9 +441,9 @@ static int http_da_init(struct soap *soap, struct http_da_data *data)
 static int http_da_copy(struct soap *soap, struct soap_plugin *dst, struct soap_plugin *src)
 {
   dst->data = (void*)SOAP_MALLOC(soap, sizeof(struct http_da_data));
-  memcpy(dst->data, src->data, sizeof(struct http_da_data));
+  soap_memcpy((void*)dst->data, sizeof(struct http_da_data), (const void*)src->data, sizeof(struct http_da_data));
   ((struct http_da_data*)dst->data)->context = NULL;
-  memset(((struct http_da_data*)dst->data)->digest, 0, sizeof(((struct http_da_data*)dst->data)->digest));
+  memset((void*)((struct http_da_data*)dst->data)->digest, 0, sizeof(((struct http_da_data*)dst->data)->digest));
   ((struct http_da_data*)dst->data)->nonce = NULL;
   ((struct http_da_data*)dst->data)->opaque = NULL;
   ((struct http_da_data*)dst->data)->qop = NULL;
@@ -445,7 +451,7 @@ static int http_da_copy(struct soap *soap, struct soap_plugin *dst, struct soap_
   ((struct http_da_data*)dst->data)->nc = 0;
   ((struct http_da_data*)dst->data)->ncount = NULL;
   ((struct http_da_data*)dst->data)->cnonce = NULL;
-  ((struct http_da_data*)dst->data)->response = NULL;
+  memset((void*)((struct http_da_data*)dst->data)->response, 0, sizeof(((struct http_da_data*)dst->data)->response));
   return SOAP_OK;
 }
 
@@ -473,7 +479,7 @@ static int http_da_post_header(struct soap *soap, const char *key, const char *v
   /* client's HTTP Authorization request */
   if (key && (!strcmp(key, "Authorization") || !strcmp(key, "Proxy-Authorization")))
   {
-    char HA1[33], entityHAhex[33], response[33];
+    char HA1[33], entityHAhex[33], response[33], responseHA[16];
     char cnonce[HTTP_DA_NONCELEN];
     char ncount[9];
     const char *qop, *method;
@@ -510,31 +516,21 @@ static int http_da_post_header(struct soap *soap, const char *key, const char *v
     else
       method = "POST";
 
-#ifdef HAVE_SNPRINTF
-    soap_snprintf(ncount, sizeof(ncount), "%8.8lx", data->nc++);
-#else
-    sprintf(ncount, "%8.8lx", data->nc++);
-#endif
+    (SOAP_SNPRINTF(ncount, sizeof(ncount), 8), "%8.8lx", data->nc++);
 
-    http_da_calc_response(soap, &data->context, HA1, data->nonce, ncount, cnonce, qop, method, soap->path, entityHAhex, response);
+    http_da_calc_response(soap, &data->context, HA1, data->nonce, ncount, cnonce, qop, method, soap->path, entityHAhex, response, responseHA);
+
+    (SOAP_SNPRINTF(soap->tmpbuf, sizeof(soap->tmpbuf), strlen(soap->authrealm) + strlen(userid) + strlen(data->nonce) + strlen(soap->path) + strlen(ncount) + strlen(cnonce) + strlen(response) + 75), "Digest realm=\"%s\", username=\"%s\", nonce=\"%s\", uri=\"%s\", nc=%s, cnonce=\"%s\", response=\"%s\"", soap->authrealm, userid, data->nonce, soap->path, ncount, cnonce, response);
 
-#ifdef HAVE_SNPRINTF
-    soap_snprintf(soap->tmpbuf, sizeof(soap->tmpbuf), "Digest realm=\"%s\", username=\"%s\", nonce=\"%s\", uri=\"%s\", nc=%s, cnonce=\"%s\", response=\"%s\"", soap->authrealm, userid, data->nonce, soap->path, ncount, cnonce, response);
-#else
-    sprintf(soap->tmpbuf, "Digest realm=\"%s\", username=\"%s\", nonce=\"%s\", uri=\"%s\", nc=%s, cnonce=\"%s\", response=\"%s\"", soap->authrealm, userid, data->nonce, soap->path, ncount, cnonce, response);
-#endif
     if (data->opaque)
-#ifdef HAVE_SNPRINTF
-      soap_snprintf(soap->tmpbuf + strlen(soap->tmpbuf), sizeof(soap->tmpbuf) - strlen(soap->tmpbuf), ", opaque=\"%s\"", data->opaque);
-#else
-      sprintf(soap->tmpbuf + strlen(soap->tmpbuf), ", opaque=\"%s\"", data->opaque);
-#endif
+    { size_t l = strlen(soap->tmpbuf);
+      (SOAP_SNPRINTF(soap->tmpbuf + l, sizeof(soap->tmpbuf) - l, strlen(data->opaque) + 11), ", opaque=\"%s\"", data->opaque);
+    }
+
     if (qop)
-#ifdef HAVE_SNPRINTF
-      soap_snprintf(soap->tmpbuf + strlen(soap->tmpbuf), sizeof(soap->tmpbuf) - strlen(soap->tmpbuf), ", qop=\"%s\"", qop);
-#else
-      sprintf(soap->tmpbuf + strlen(soap->tmpbuf), ", qop=\"%s\"", qop);
-#endif
+    { size_t l = strlen(soap->tmpbuf);
+      (SOAP_SNPRINTF(soap->tmpbuf + l, sizeof(soap->tmpbuf) - l, strlen(qop) + 8), ", qop=\"%s\"", qop);
+    }
 
     return data->fposthdr(soap, key, soap->tmpbuf);
   }
@@ -550,11 +546,7 @@ static int http_da_post_header(struct soap *soap, const char *key, const char *v
 
     http_da_session_start(soap->authrealm, nonce, opaque);
 
-#ifdef HAVE_SNPRINTF
-    soap_snprintf(soap->tmpbuf, sizeof(soap->tmpbuf), "Digest realm=\"%s\", qop=\"auth,auth-int\", nonce=\"%s\", opaque=\"%s\"", soap->authrealm, nonce, opaque);
-#else
-    sprintf(soap->tmpbuf, "Digest realm=\"%s\", qop=\"auth,auth-int\", nonce=\"%s\", opaque=\"%s\"", soap->authrealm, nonce, opaque);
-#endif
+    (SOAP_SNPRINTF(soap->tmpbuf, sizeof(soap->tmpbuf), strlen(soap->authrealm) + strlen(nonce) + strlen(opaque) + 59), "Digest realm=\"%s\", qop=\"auth,auth-int\", nonce=\"%s\", opaque=\"%s\"", soap->authrealm, nonce, opaque);
 
     return data->fposthdr(soap, key, soap->tmpbuf);
   }
@@ -615,7 +607,7 @@ static int http_da_parse_header(struct soap *soap, const char *key, const char *
     data->alg = NULL;
     data->ncount = soap_strdup(soap, soap_get_header_attribute(soap, val + 7, "nc"));
     data->cnonce = soap_strdup(soap, soap_get_header_attribute(soap, val + 7, "cnonce"));
-    data->response = soap_strdup(soap, soap_get_header_attribute(soap, val + 7, "response"));
+    soap_hex2s(soap, soap_get_header_attribute(soap, val + 7, "response"), data->response, 16, NULL);
     return SOAP_OK;
   }
 
@@ -630,7 +622,6 @@ static int http_da_parse_header(struct soap *soap, const char *key, const char *
     data->nc = 1;
     data->ncount = NULL;
     data->cnonce = NULL;
-    data->response = NULL;
     return SOAP_OK;
   }
 
@@ -887,7 +878,7 @@ int http_da_verify_get(struct soap *soap, const char *passwd)
 static int http_da_verify_method(struct soap *soap, const char *method, const char *passwd)
 {
   struct http_da_data *data = (struct http_da_data*)soap_lookup_plugin(soap, http_da_id);
-  char HA1[33], entityHAhex[33], response[33];
+  char HA1[33], entityHAhex[33], response[33], responseHA[16];
 
   if (!data)
     return SOAP_ERR;
@@ -910,14 +901,10 @@ static int http_da_verify_method(struct soap *soap, const char *method, const ch
   if (!soap_tag_cmp(data->qop, "auth-int"))
     soap_s2hex(soap, (unsigned char*)data->digest, entityHAhex, 16);
 
-  http_da_calc_response(soap, &data->context, HA1, data->nonce, data->ncount, data->cnonce, data->qop, method, soap->path, entityHAhex, response);
-
-#ifdef SOAP_DEBUG
-  fprintf(stderr, "Debug message: verifying client response=%s with calculated digest=%s qop=%s\n", data->response, response, data->qop);
-#endif
+  http_da_calc_response(soap, &data->context, HA1, data->nonce, data->ncount, data->cnonce, data->qop, method, soap->path, entityHAhex, response, responseHA);
 
   /* check digest response values */
-  if (strcmp(data->response, response))
+  if (memcmp(data->response, responseHA, 16))
     return SOAP_ERR;
 
   return SOAP_OK;
@@ -1047,20 +1034,14 @@ static void http_da_session_cleanup()
 void http_da_calc_nonce(struct soap *soap, char nonce[HTTP_DA_NONCELEN])
 {
   static short count = 0xCA53;
-#ifdef HAVE_SNPRINTF
-  soap_snprintf(nonce, sizeof(nonce), "%8.8x%4.4hx%8.8x", (int)time(NULL), count++, soap_random);
-#else
-  sprintf(nonce, "%8.8x%4.4hx%8.8x", (int)time(NULL), count++, soap_random);
-#endif
+  (void)soap;
+  (SOAP_SNPRINTF(nonce, HTTP_DA_NONCELEN, 20), "%8.8x%4.4hx%8.8x", (int)time(NULL), count++, soap_random);
 }
 
 void http_da_calc_opaque(struct soap *soap, char opaque[HTTP_DA_OPAQUELEN])
 {
-#ifdef HAVE_SNPRINTF
-  soap_snprintf(opaque, sizeof(opaque), "%8.8x", soap_random);
-#else
-  sprintf(opaque, "%8.8x", soap_random);
-#endif
+  (void)soap;
+  (SOAP_SNPRINTF(opaque, HTTP_DA_OPAQUELEN, 8), "%8.8x", soap_random);
 }
 
 /******************************************************************************\
@@ -1098,9 +1079,9 @@ static void http_da_calc_HA1(struct soap *soap, void **context, const char *alg,
   soap_s2hex(soap, (unsigned char*)HA1, HA1hex, 16);
 };
 
-static void http_da_calc_response(struct soap *soap, void **context, char HA1hex[33], const char *nonce, const char *ncount, const char *cnonce, const char *qop, const char *method, const char *uri, char entityHAhex[33], char response[33])
+static void http_da_calc_response(struct soap *soap, void **context, char HA1hex[33], const char *nonce, const char *ncount, const char *cnonce, const char *qop, const char *method, const char *uri, char entityHAhex[33], char response[33], char responseHA[16])
 {
-  char HA2[16], HA2hex[33], responseHA[16];
+  char HA2[16], HA2hex[33];
 
   md5_handler(soap, context, MD5_INIT, NULL, 0);
   md5_handler(soap, context, MD5_UPDATE, (char*)method, strlen(method));
