@@ -170,9 +170,9 @@ static int http_get_init(struct soap *soap, struct http_get_data *data, int (*ha
   data->stat_get = 0;
   data->stat_post = 0;
   data->stat_fail = 0;
-  memset(data->min, 0, sizeof(data->min));
-  memset(data->hour, 0, sizeof(data->hour));
-  memset(data->day, 0, sizeof(data->day));
+  memset((void*)data->min, 0, sizeof(data->min));
+  memset((void*)data->hour, 0, sizeof(data->hour));
+  memset((void*)data->day, 0, sizeof(data->day));
   soap->fparse = http_get_parse; /* replace HTTP header parser callback with ours */
   return SOAP_OK;
 }
@@ -185,7 +185,8 @@ static int http_get_copy(struct soap *soap, struct soap_plugin *dst, struct soap
 */
 
 static void http_get_delete(struct soap *soap, struct soap_plugin *p)
-{ free(p->data); /* free allocated plugin data (this function is not called for shared plugin data, but only when the final soap_done() is invoked on the original soap struct) */
+{ (void)soap;
+  free(p->data); /* free allocated plugin data (this function is not called for shared plugin data, but only when the final soap_done() is invoked on the original soap struct) */
 }
 
 static int http_get_parse(struct soap *soap)
@@ -247,6 +248,7 @@ char *query(struct soap *soap)
 
 char *query_key(struct soap *soap, char **s)
 { char *t = *s;
+  (void)soap;
   if (t && *t)
   { *s = (char*)soap_decode_string(t, strlen(t), t + 1);
     return t;
@@ -256,6 +258,7 @@ char *query_key(struct soap *soap, char **s)
 
 char *query_val(struct soap *soap, char **s)
 { char *t = *s;
+  (void)soap;
   if (t && *t == '=')
   { *s = (char*)soap_decode_string(t, strlen(t), t + 1);
     return t;
@@ -264,8 +267,8 @@ char *query_val(struct soap *soap, char **s)
 }
 
 int soap_encode_string(const char *s, char *t, size_t len)
-{ register int c;
-  register size_t n = len;
+{ int c;
+  size_t n = len;
   while ((c = *s++) && n-- > 1)
   { if (c == ' ') 
       *t++ = '+';
