@@ -81,7 +81,7 @@ compiling, linking, and/or using OpenSSL is allowed.
 	soap_done(&soap); // detach plugin (calls plugin->fdelete)
 
 	A POST handler function is triggered by the media type in the
-	http_post_handlers table. Use http_copy_body() as below to retrieve
+	http_post_handlers table. Use http_get_http_body() as below to retrieve
 	HTTP POST body data:
 
 	int image_handler(struct soap *soap)
@@ -257,14 +257,14 @@ int soap_delete_connect(struct soap *soap, const char *endpoint, const char *act
 
 int soap_http_body(struct soap *soap, char **buf, size_t *len)
 { char *s = *buf = NULL;
-  *len = 0;
   /* It is unlikely chunked and/or compressed POST messages are sent by browsers, but we need to handle them */
   if ((soap->mode & SOAP_IO) == SOAP_IO_CHUNK
 #ifdef WITH_ZLIB
    || soap->zlib_in != SOAP_ZLIB_NONE
 #endif
    )
-  { register size_t k;
+  { size_t k;
+    size_t n;
     soap_wchar c = EOF;
     soap->labidx = 0;
     do
@@ -279,9 +279,11 @@ int soap_http_body(struct soap *soap, char **buf, size_t *len)
         *s++ = c;
       }
     } while (c != (int)EOF);
-    *len = soap->lablen - k - 1;
-    *buf = (char*)soap_malloc(soap, *len + 1);
-    memcpy(*buf, soap->labbuf, *len + 1);
+    n = soap->lablen - k - 1;
+    *buf = (char*)soap_malloc(soap, n + 1);
+    soap_memcpy(*buf, n + 1, soap->labbuf, n + 1);
+    if (len)
+      *len = n;
   }
   else
   { if (soap->length)
@@ -299,9 +301,10 @@ int soap_http_body(struct soap *soap, char **buf, size_t *len)
       }
     }
     *buf = s;
-    *len = soap->length;
+    if (len)
+      *len = soap->length;
   }
-  return soap_end_recv(soap);
+  return SOAP_OK;
 }
 
 /******************************************************************************/
