@@ -4,7 +4,7 @@
 	WS-Addressing plugin.
 
 gSOAP XML Web services tools
-Copyright (C) 2000-2008, Robert van Engelen, Genivia Inc., All Rights Reserved.
+Copyright (C) 2000-2015, Robert van Engelen, Genivia Inc., All Rights Reserved.
 This part of the software is released under one of the following licenses:
 GPL, the gSOAP public license, or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -19,7 +19,7 @@ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the License.
 
 The Initial Developer of the Original Code is Robert A. van Engelen.
-Copyright (C) 2000-2008, Robert van Engelen, Genivia Inc., All Rights Reserved.
+Copyright (C) 2000-2015, Robert van Engelen, Genivia Inc., All Rights Reserved.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -57,7 +57,9 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 /**
 
-@page wsa_0 The wsa plugin for client applications and stand-alone services
+@page wsa_0 The WS-Addressing plugin
+
+[TOC]
 
 @section wsa_1 WS-Addressing Setup
 
@@ -75,9 +77,9 @@ To use the wsa plugin:
    wsaapi.c and the soapcpp2-generated source files.
 -# Use the wsa plugin API functions described below.
 
-An example wsa client/server application can be found in samples/wsa.
+An example wsa client/server application can be found in `gsoap/samples/wsa`.
 
-A gSOAP service definitions header file with a "wsa import" to support
+A gSOAP service definitions header file with a `#import "wsa.h"` to support
 WS-Addressing is automatically generated by wsdl2h for a set of WSDLs that use
 WS-Addressing. The wsdl2h-generated header file should be further processed by
 soapcpp2 to generate the binding code. The wsaapi.h and wsaapi.c implement the
@@ -87,8 +89,8 @@ A wsdl2h-generated service definitions header file might include the following
 imports:
 
 @code
-#import "soap12.h"
-#import "wsa.h" // or wsa3.h (2003/03), wsa4.h (2004/03), wsa5.h (2005/03)
+    #import "soap12.h"
+    #import "wsa.h" // or wsa3.h (2003/03), wsa4.h (2004/03), wsa5.h (2005/03)
 @endcode
 
 The wsa.h header file is imported from import/wsa.h when soapcpp2 is run on
@@ -105,9 +107,7 @@ For developers: the WS-Addressing header blocks in wsa.h (and others) were
 generated from the WS-Addressing schema with the wsdl2h tool and
 WS/WS-typemap.dat as follows:
 
-@code
-    > wsdl2h -cegy -o wsa.h -t WS/WS-typemap.dat WS/WS-Addressing.xsd
-@endcode
+    wsdl2h -cegy -o wsa.h -t WS/WS-typemap.dat WS/WS-Addressing.xsd
 
 Refer to wsa.h for more details.
 
@@ -116,83 +116,82 @@ Refer to wsa.h for more details.
 @subsection wsa_2_1 Constructing WS-Addressing Information Headers
 
 To associate WS-Addressing information headers with service operations, the
-SOAP Header struct SOAP_ENV__Header must have been defined and for each service
+SOAP Header struct `SOAP_ENV__Header` must have been defined and for each service
 operation that uses WS-Addressing method-header-part directives should be used
 in the gSOAP service definitions header file as follows:
 
 @code
-#import "wsa.h"
-
-//gsoap ns service method-header-part: example wsa__MessageID
-//gsoap ns service method-header-part: example wsa__RelatesTo
-//gsoap ns service method-header-part: example wsa__From
-//gsoap ns service method-header-part: example wsa__ReplyTo
-//gsoap ns service method-header-part: example wsa__FaultTo
-//gsoap ns service method-header-part: example wsa__To
-//gsoap ns service method-header-part: example wsa__Action
-//gsoap ns service method-action: example urn:example/examplePort/example
-int ns__example(char *in, struct ns__exampleResponse *out);
+    #import "wsa.h"
+    //gsoap ns service method-header-part: example wsa__MessageID
+    //gsoap ns service method-header-part: example wsa__RelatesTo
+    //gsoap ns service method-header-part: example wsa__From
+    //gsoap ns service method-header-part: example wsa__ReplyTo
+    //gsoap ns service method-header-part: example wsa__FaultTo
+    //gsoap ns service method-header-part: example wsa__To
+    //gsoap ns service method-header-part: example wsa__Action
+    //gsoap ns service method-action: example urn:example/examplePort/example
+    int ns__example(char *in, struct ns__exampleResponse *out);
 @endcode
 
-Note that the use of wsa versions determines the wsa prefix, e.g. use wsa5 for
-the latest WS-Addressing as in wsa5__MessageID.
+Note that the use of wsa versions determines the wsa prefix, e.g. use `wsa5`
+for the latest WS-Addressing as in `wsa5__MessageID`.
 
 In the client-side code, the WS-Addressing information headers are set with
-soap_wsa_request() by passing an optional message UUID string, a mandatory
+`soap_wsa_request` by passing an optional message UUID string, a mandatory
 destination address URI string, and a mandatory request action URI string. The
 wsa plugin should be registered with the current soap struct context. An
 optional source address information header can be added with
-soap_wsa_add_From() (must be invoked after the soap_wsa_request call).
+`soap_wsa_add_From` (must be invoked after the `soap_wsa_request` call).
 
 For example:
 
 @code
-#include "wsaapi.h"
-soap_register_plugin(soap, soap_wsa);
+    #include "wsaapi.h"
+    soap_register_plugin(soap, soap_wsa);
 
-if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction))
- || soap_wsa_add_From(soap, FromAddress)) // optional: add a 'From' address
-  ... // error: out of memory
+    if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction))
+     || soap_wsa_add_From(soap, FromAddress)) // optional: add a 'From' address
+      ... // error: out of memory
 
-if (soap_call_ns__example(soap, ToAddress, NULL, ...))
-  soap_print_fault(soap, stderr); // an error occurred
-else
-  // process the response 
+    if (soap_call_ns__example(soap, ToAddress, NULL, ...))
+      soap_print_fault(soap, stderr); // an error occurred
+    else
+      // process the response 
 @endcode
 
 To generate a UUID for the RequestMessageID, use:
 
 @code
-const char *RequestMessageID = soap_wsa_rand_uuid(soap);
+    const char *RequestMessageID = soap_wsa_rand_uuid(soap);
 @endcode
 
 @subsection wsa_2_2 Information Headers for Relaying Server Responses
 
 To relay the response to another destination, the WS-Addressing ReplyTo
-information header is added with soap_wsa_add_ReplyTo() by passing a reply
-address URI string. The service returns HTTP 202 ACCEPTED to the client when
+information header is added with `soap_wsa_add_ReplyTo` by passing a reply
+address URI string. The service returns "HTTP 202 ACCEPTED" to the client when
 the response message relay was successful.
 
 For example:
 
 @code
-#include "wsaapi.h"
-soap_register_plugin(soap, soap_wsa);
+    #include "wsaapi.h"
+    soap_register_plugin(soap, soap_wsa);
 
-if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
- || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
- || soap_wsa_add_ReplyTo(soap, ReplyToAddress))
-  ... // error: out of memory
+    if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
+     || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
+     || soap_wsa_add_ReplyTo(soap, ReplyToAddress))
+      ... // error: out of memory
 
-if (soap_call_ns__example(soap, ToAddress, NULL, ...))
-{
-  if (soap->error == 202) // HTTP ACCEPTED
-    printf("Request was accepted and results were forwarded\n");
-  else
-    soap_print_fault(soap, stderr); // an error occurred
-}
-else
-  // unexpected OK: for some reason the response was not relayed
+    if (soap_call_ns__example(soap, ToAddress, NULL, ...))
+    {
+      if (soap->error == 202) // HTTP ACCEPTED
+	printf("Request was accepted and results were forwarded\n");
+      else
+	soap_print_fault(soap, stderr); // an error occurred
+    }
+    else
+      // unexpected OK: for some reason the response was not relayed
 @endcode
 
 Note: the response message will be relayed when the From address is absent or
@@ -201,68 +200,68 @@ different than the ReplyTo address
 @subsection wsa_2_3 Information Headers for Relaying Server Faults
 
 To relay a server fault message to another destination, the WS-Addressing
-FaultTo information header is added with soap_wsa_add_FaultTo() by passing a
-relay address URI string. The service returns HTTP 202 ACCEPTED to the client
+FaultTo information header is added with `soap_wsa_add_FaultTo` by passing a
+relay address URI string. The service returns "HTTP 202 ACCEPTED" to the client
 when the fault was relayed.
 
 For example:
 
 @code
-#include "wsaapi.h"
-soap_register_plugin(soap, soap_wsa);
+    #include "wsaapi.h"
+    soap_register_plugin(soap, soap_wsa);
 
-if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
- || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
- || soap_wsa_add_FaultTo(soap, FaultToAddress))
-  ... // error: out of memory
+    if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
+     || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
+     || soap_wsa_add_FaultTo(soap, FaultToAddress))
+      ... // error: out of memory
 
-if (soap_call_ns__example(soap, ToAddress, NULL, ...))
-{
-  if (soap->error == 202) // HTTP ACCEPTED
-    printf("A fault occurred and the fault details were forwarded\n");
-  else
-    soap_print_fault(soap, stderr); // a connection error occurred
-}
-else
-  // process response 
+    if (soap_call_ns__example(soap, ToAddress, NULL, ...))
+    {
+      if (soap->error == 202) // HTTP ACCEPTED
+	printf("A fault occurred and the fault details were forwarded\n");
+      else
+	soap_print_fault(soap, stderr); // a connection error occurred
+    }
+    else
+      ... // process response 
 @endcode
 
 Note that the call can still return a fault, such as a connection error when
 the service is not responding. In addition to the fault relay, the responses
-can be relayed with soap_wsa_add_ReplyTo().
+can be relayed with `soap_wsa_add_ReplyTo`.
 
 @subsection wsa_2_4 Error Handling
 
 SOAP and HTTP errors set the soap->error attribute, as shown in this example:
 
 @code
-if (soap_call_ns__example(soap, ToAddress, NULL, ...))
-{
-  if (soap->error == 202) // HTTP ACCEPTED
-    printf("A fault occurred and the fault details were forwarded\n");
-  else
-    soap_print_fault(soap, stderr); // a connection error occurred
-}
-else
-  // process response 
+    if (soap_call_ns__example(soap, ToAddress, NULL, ...))
+    {
+      if (soap->error == 202) // HTTP ACCEPTED
+	printf("A fault occurred and the fault details were forwarded\n");
+      else
+	soap_print_fault(soap, stderr); // a connection error occurred
+    }
+    else
+      ... // process response 
 @endcode
 
 When a WS-Addressing error occurred, the wsa error code is stored in the SOAP
 Fault Subcode field. This information can be retrieved with:
 
 @code
-wsa__FaultSubcodeValues fault;
-if (soap_wsa_check_fault(soap, &fault))
-{
-  switch (fault)
-  {
-    case wsa__InvalidMessageInformationHeader: ...
-    case wsa__MessageInformationHeaderRequired: ...
-    case wsa__DestinationUreachable: ...
-    case wsa__ActionNotSupported: ...
-    case wsa__EndpointUnavailable: ...
-  }
-}
+    wsa__FaultSubcodeValues fault;
+    if (soap_wsa_check_fault(soap, &fault))
+    {
+      switch (fault)
+      {
+	case wsa__InvalidMessageInformationHeader: ...
+	case wsa__MessageInformationHeaderRequired: ...
+	case wsa__DestinationUreachable: ...
+	case wsa__ActionNotSupported: ...
+	case wsa__EndpointUnavailable: ...
+      }
+    }
 @endcode
 
 When using wsa5.h, please refer to the standards and fault codes for this
@@ -270,108 +269,110 @@ implementation. For the wsa5.h 2005/03 standard, several faults have an
 additional parameter (SOAP Fault detail):
 
 @code
-wsa5__FaultCodesType fault;
-char *info;
-if (soap_wsa_check_fault(soap, &fault, &info))
-{
-  switch (fault)
-  {
-    case wsa5__InvalidAddressingHeader:
-      if (info)
-        printf("The invalid addressing header element is %s\n", info);
-    ...
-  }
-}
+    wsa5__FaultCodesType fault;
+    char *info;
+    if (soap_wsa_check_fault(soap, &fault, &info))
+    {
+      switch (fault)
+      {
+	case wsa5__InvalidAddressingHeader:
+	  if (info)
+	    printf("The invalid addressing header element is %s\n", info);
+	...
+      }
+    }
 @endcode
 
 @subsection wsa_2_5 Combining WS-Addressing with WS-Security
 
 WS-Security can be combined with WS-Addressing. To sign WS-Addressing header
-blocks, use the soap_wsse_set_wsu_id() WSSE-plugin call to set the wsu:Id
+blocks, use the `soap_wsse_set_wsu_id` WSSE-plugin call to set the wsu:Id
 attribute and signing of these attributed elements. For example, suppose we use
 WS-Addressing 2005:
 
 @code
-#include "wsaapi.h"
-#include "wsseapi.h"
-soap_register_plugin(soap, soap_wsa);
-soap_register_plugin(soap, soap_wsse);
+    #include "wsaapi.h"
+    #include "wsseapi.h"
+    soap_register_plugin(soap, soap_wsa);
+    soap_register_plugin(soap, soap_wsse);
 
-soap_wsse_set_wsu_id(soap, "wsa5:From wsa5:To wsa5:ReplyTo wsa5:FaultTo wsa5:Action");
-if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
- || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
- || soap_wsa_add_FaultTo(soap, FaultToAddress))
-  ... // error: out of memory
-if (soap_call_ns__example(soap, ToAddress, NULL, ...))
-  ... // error
+    soap_wsse_set_wsu_id(soap, "wsa5:From wsa5:To wsa5:ReplyTo wsa5:FaultTo wsa5:Action");
+    if (soap_wsa_request(soap, RequestMessageID, ToAddress, RequestAction)
+     || soap_wsa_add_From(soap, FromAddress) // optional: add a 'From' address
+     || soap_wsa_add_FaultTo(soap, FaultToAddress))
+      ... // error: out of memory
+    if (soap_call_ns__example(soap, ToAddress, NULL, ...))
+      ... // error
 @endcode
 
-Note: soap_wsse_set_wsu_id() should only be set once. Each new call overrides
+Note: `soap_wsse_set_wsu_id` should only be set once. Each new call overrides
 the previous.
 
 For more details on WS-Security, please see the
-<a href="../../wsse/html/index.html">WSSE plugin documentation</a>.
+[WS-Security plugin documentation](../../wsse/html/index.html).
 
 @section wsa_3 Server-side Usage
 
 The wsa plugin should be registered with:
 
 @code
-soap_register_plugin(soap, soap_wsa);
+    soap_register_plugin(soap, soap_wsa);
 @endcode
 
-Once the plugin is registered, the soap_bind(), soap_accept(), and soap_serve()
+Once the plugin is registered, the `soap_bind`, `soap_accept`, and `soap_serve`
 functions can be called to process requests and semi-automatically handle the
 WS-Addressing header blocks.
 
 Important: to dispatch service operations based on the WS-Addressing wsa:Action
-information header, you must use soapcpp2 option -a. The generates a new
+information header, you must use soapcpp2 option `-a`. The generates a new
 dispatcher (in soapServer.c) based on the action value.
 
-A service operation implementation should use soap_wsa_check() at the start of
+A service operation implementation should use `soap_wsa_check` at the start of
 its execution to verify the validity of the WS-Addressing information headers
 in the SOAP request message. To allow response message to be automatically
 relayed based on the ReplyTo information header, the service operation should
-return soap_wsa_reply() with an optional message UUID string and a mandatory
+return `soap_wsa_reply` with an optional message UUID string and a mandatory
 response action string. The response action string is documented in the
 wsdl2h-generated .h file for this service operation.
 
 For example:
 
 @code
-int ns__example(struct soap *soap, char *in, struct ns__exampleResponse *out)
-{ if (soap_wsa_check(soap))
-    return soap->error;
-  // ... service logic
-  return soap_wsa_reply(soap, ResponseMessageID, ResponseAction);
-}
+    int ns__example(struct soap *soap, char *in, struct ns__exampleResponse *out)
+    { if (soap_wsa_check(soap))
+	return soap->error;
+      // ... service logic
+      return soap_wsa_reply(soap, ResponseMessageID, ResponseAction);
+    }
 @endcode
 
 To return a SOAP fault that is automatically relayed to a fault service based
-on the FaultTo information header, the soap_wsa_sender_fault(),
-soap_wsa_receiver_fault(), soap_wsa_sender_fault_subcode(), and
-soap_wsa_receiver_fault_subcode() functions should be used instead of the
-usual soap_sender_fault(), soap_receiver_fault(), soap_sender_fault_subcode(),
-and soap_receiver_fault_subcode(), respectively. In case a Action must be
-associated with a SOAP Fault, use the soap_wsa_sender_fault_subcode_action()
-and soap_wsa_receiver_fault_subcode_action() functions to set the WS-Addressing
+on the FaultTo information header, the `soap_wsa_sender_fault`,
+`soap_wsa_receiver_fault`, `soap_wsa_sender_fault_subcode`, and
+`soap_wsa_receiver_fault_subcode` functions should be used instead of the
+usual `soap_sender_fault`, `soap_receiver_fault`, `soap_sender_fault_subcode`,
+and `soap_receiver_fault_subcode`, respectively:
+  
+In case a Action must be associated with a SOAP Fault, use the
+`soap_wsa_sender_fault_subcode_action` and
+`soap_wsa_receiver_fault_subcode_action` functions to set the WS-Addressing
 Action (and HTTP SOAP Action header as well).
 
 For example, the following service operation illustrates the use of
-soap_wsa_check() to verify and process WS-Addressing header blocks and
-soap_wsA_reply() to enable responses to be relayed as per ReplyTo address in
+`soap_wsa_check` to verify and process WS-Addressing header blocks and
+`soap_wsa_reply` to enable responses to be relayed as per ReplyTo address in
 the WS-Addressing header:
 
 @code
-int ns__example(struct soap *soap, char *in, struct ns__exampleResponse *out)
-{ if (soap_wsa_check(soap))
-    return soap->error;
-  // ... service logic
-  // ... an error occurred, need to return fault possibly to fault service:
-    return soap_wsa_sender_fault(soap, "Exception in service operation", NULL);
-  // ... normal execution continues
-  return soap_wsa_reply(soap, ResponseMessageID, ResponseAction);
-}
+    int ns__example(struct soap *soap, char *in, struct ns__exampleResponse *out)
+    { if (soap_wsa_check(soap))
+	return soap->error;
+      // ... service logic
+      // ... an error occurred, need to return fault possibly to fault service:
+	return soap_wsa_sender_fault(soap, "Exception in service operation", NULL);
+      // ... normal execution continues
+      return soap_wsa_reply(soap, ResponseMessageID, ResponseAction);
+    }
 @endcode
 
 @section wsa_4 HTTPS Server-side Usage
@@ -381,28 +382,28 @@ link with libgsoapssl, libssl, and libcrypto). Because WS-Addressing may relay
 messages over HTTPS as a sender (client), you must initialize the SSL context
 for server and client uses. Therefore, the context must have access to all the
 certificates need to verify the authenticity of the ReplyTo and FaultTo HTTPS
-servers. To do so, use the following SSL initialization before soap_bind():
+servers. To do so, use the following SSL initialization before `soap_bind`:
 
 @code
-struct soap *soap = soap_new();
-if (soap_ssl_server_context(soap,
-  SOAP_SSL_DEFAULT,
-  "server.pem", // the keyfile (server should authenticate)
-  "password",   // password to read the key file
-  "cacert.pem", // cacert file to store trusted certificates (role as client)
-  NULL,         // optional capath
-  NULL,         // DH file name or DH param key len bits, when NULL use RSA
-  NULL,         // file with random data to seed randomness
-  "myserver"    // unique server identification for SSL session cache
-))
-{ soap_print_fault(soap, stderr);
-  ...
-}
-soap->bind_flags = SO_REUSEADDR;
-if (!soap_valid_socket(soap_bind(soap, NULL, port, 100)))
-{ soap_print_fault(soap, stderr);
-  ...
-}
+    struct soap *soap = soap_new();
+    if (soap_ssl_server_context(soap,
+      SOAP_SSL_DEFAULT,
+      "server.pem", // the keyfile (server should authenticate)
+      "password",   // password to read the key file
+      "cacert.pem", // cacert file to store trusted certificates (role as client)
+      NULL,         // optional capath
+      NULL,         // DH file name or DH param key len bits, when NULL use RSA
+      NULL,         // file with random data to seed randomness
+      "myserver"    // unique server identification for SSL session cache
+    ))
+    { soap_print_fault(soap, stderr);
+      ...
+    }
+    soap->bind_flags = SO_REUSEADDR;
+    if (!soap_valid_socket(soap_bind(soap, NULL, port, 100)))
+    { soap_print_fault(soap, stderr);
+      ...
+    }
 @endcode
 
 @section wsa_5 Implementing a Server for Handling ReplyTo Response Messages
@@ -411,40 +412,39 @@ To implement a separate server for handling relayed SOAP response messages
 based on the ReplyTo information header in the request message, the gSOAP
 header file should include a one-way service operation for the response
 message. These one-way response service operations are automatically generated
-with wsdl2h option -b.
+with wsdl2h option `-b`.
 
 For example, suppose a service operation returns an exampleResponse message. We
 declare the one-way exampleResponse operation as follows:
 
 @code
-#import "wsa.h"
-
-//gsoap ns service method-header-part: exampleResult wsa__MessageID
-//gsoap ns service method-header-part: exampleResult wsa__RelatesTo
-//gsoap ns service method-header-part: exampleResult wsa__From
-//gsoap ns service method-header-part: exampleResult wsa__ReplyTo
-//gsoap ns service method-header-part: exampleResult wsa__FaultTo
-//gsoap ns service method-header-part: exampleResult wsa__To
-//gsoap ns service method-header-part: exampleResult wsa__Action
-//gsoap ns service method-action: exampleResult urn:example/examplePort/exampleResponse
-int ns__exampleResponse(char *out, void);
+    #import "wsa.h"
+    //gsoap ns service method-header-part: exampleResult wsa__MessageID
+    //gsoap ns service method-header-part: exampleResult wsa__RelatesTo
+    //gsoap ns service method-header-part: exampleResult wsa__From
+    //gsoap ns service method-header-part: exampleResult wsa__ReplyTo
+    //gsoap ns service method-header-part: exampleResult wsa__FaultTo
+    //gsoap ns service method-header-part: exampleResult wsa__To
+    //gsoap ns service method-header-part: exampleResult wsa__Action
+    //gsoap ns service method-action: exampleResult urn:example/examplePort/exampleResponse
+    int ns__exampleResponse(char *out, void);
 @endcode
 
 Note that the action information is important, because it is used by the
-service dispatcher (assuming soapcpp2 option -a is used).
+service dispatcher (assuming soapcpp2 option `-a` is used).
 
 The implementation in the server code uses soap_wsa_check() to check the
 presense and validity of the WS-Addressing information header in the message.
-The soap_send_empty_response() function should be used to return an
-acknowledgment HTTP header with HTTP 202 ACCEPTED to the sender:
+The `soap_send_empty_response` function should be used to return an
+acknowledgment HTTP header with "HTTP 202 ACCEPTED" to the sender:
 
 @code
-int ns__exampleResponse(struct soap *soap, char *out)
-{ if (soap_wsa_check(soap))
-    return soap_send_empty_response(soap, 500); // HTTP 500 Internal Server Error
-  // ... service logic
-  return soap_send_empty_response(soap, SOAP_OK); // HTTP 202 ACCEPTED
-}
+    int ns__exampleResponse(struct soap *soap, char *out)
+    { if (soap_wsa_check(soap))
+	return soap_send_empty_response(soap, 500); // HTTP 500 Internal Server Error
+      // ... service logic
+      return soap_send_empty_response(soap, SOAP_OK); // HTTP 202 ACCEPTED
+    }
 @endcode
 
 @section wsa_6 Implementing a Server for Handling FaultTo Fault Messages
@@ -463,44 +463,44 @@ The fault operation in the WS-Addressing files (wsa5.h etc.) is declared as
 follows (here shown for the 2004/08 standard):
 
 @code
-//gsoap SOAP_ENV service method-action: Fault http://schemas.xmlsoap.org/ws/2004/08/addressing/fault
-int SOAP_ENV__Fault
-(       _QName			 faultcode,		// SOAP 1.1
-        char			*faultstring,		// SOAP 1.1
-        char			*faultactor,		// SOAP 1.1
-        struct SOAP_ENV__Detail	*detail,		// SOAP 1.1
-        struct SOAP_ENV__Code	*SOAP_ENV__Code,	// SOAP 1.2
-        struct SOAP_ENV__Reason	*SOAP_ENV__Reason,	// SOAP 1.2
-        char			*SOAP_ENV__Node,	// SOAP 1.2
-        char			*SOAP_ENV__Role,	// SOAP 1.2
-        struct SOAP_ENV__Detail	*SOAP_ENV__Detail,	// SOAP 1.2
-	void
-);
+    //gsoap SOAP_ENV service method-action: Fault http://schemas.xmlsoap.org/ws/2004/08/addressing/fault
+    int SOAP_ENV__Fault
+    (       _QName			 faultcode,		// SOAP 1.1
+	    char			*faultstring,		// SOAP 1.1
+	    char			*faultactor,		// SOAP 1.1
+	    struct SOAP_ENV__Detail	*detail,		// SOAP 1.1
+	    struct SOAP_ENV__Code	*SOAP_ENV__Code,	// SOAP 1.2
+	    struct SOAP_ENV__Reason	*SOAP_ENV__Reason,	// SOAP 1.2
+	    char			*SOAP_ENV__Node,	// SOAP 1.2
+	    char			*SOAP_ENV__Role,	// SOAP 1.2
+	    struct SOAP_ENV__Detail	*SOAP_ENV__Detail,	// SOAP 1.2
+	    void
+    );
 @endcode
 
 Because each service operation has a struct to hold its input parameters, we
-automatically generate the (original) SOAP_ENV__Fault struct on the fly!
+automatically generate the (original) `SOAP_ENV__Fault` struct on the fly!
 
 It is important to associate the wsa fault action with this operation as shown
 above.
 
 The implementation of the Fault service operation in your server code should be
-something like:
+similar to:
 
 @code
-int SOAP_ENV__Fault(struct soap *soap, char *faultcode, char *faultstring, char *faultactor, struct SOAP_ENV__Detail *detail, struct SOAP_ENV__Code *SOAP_ENV__Code, struct SOAP_ENV__Reason *SOAP_ENV__Reason, char *SOAP_ENV__Node, char *SOAP_ENV__Role, struct SOAP_ENV__Detail *SOAP_ENV__Detail)
-{ 
-  ... = faultcode; // SOAP 1.1 fault code string (QName)
-  ... = faultstring; // SOAP 1.1 fault string
-  ... = faultactor; // SOAP 1.1 fault actor string
-  ... = detail; // SOAP 1.1 fault detail struct
-  ... = SOAP_ENV__Code; // SOAP 1.2 fault code struct
-  ... = SOAP_ENV__Reason; // SOAP 1.2 reason struct
-  ... = SOAP_ENV__Node; // SOAP 1.2 node string
-  ... = SOAP_ENV__Role; // SOAP 1.2 role string
-  ... = SOAP_ENV__Detail; // SOAP 1.2 detail struct
-  return SOAP_OK;
-}
+    int SOAP_ENV__Fault(struct soap *soap, char *faultcode, char *faultstring, char *faultactor, struct SOAP_ENV__Detail *detail, struct SOAP_ENV__Code *SOAP_ENV__Code, struct SOAP_ENV__Reason *SOAP_ENV__Reason, char *SOAP_ENV__Node, char *SOAP_ENV__Role, struct SOAP_ENV__Detail *SOAP_ENV__Detail)
+    { 
+      ... = faultcode; // SOAP 1.1 fault code string (QName)
+      ... = faultstring; // SOAP 1.1 fault string
+      ... = faultactor; // SOAP 1.1 fault actor string
+      ... = detail; // SOAP 1.1 fault detail struct
+      ... = SOAP_ENV__Code; // SOAP 1.2 fault code struct
+      ... = SOAP_ENV__Reason; // SOAP 1.2 reason struct
+      ... = SOAP_ENV__Node; // SOAP 1.2 node string
+      ... = SOAP_ENV__Role; // SOAP 1.2 role string
+      ... = SOAP_ENV__Detail; // SOAP 1.2 detail struct
+      return SOAP_OK;
+    }
 @endcode
 
 Note that SOAP 1.1 or SOAP 1.2 parameters are set based on the 1.1/1.2
@@ -603,11 +603,7 @@ soap_wsa_rand_uuid(struct soap *soap)
 #endif
   r3 = soap_random;
   r4 = soap_random;
-#ifdef HAVE_SNPRINTF
-  soap_snprintf(uuid, uuidlen, "urn:uuid:%8.8x-%4.4hx-4%3.3hx-%4.4hx-%4.4hx%8.8x", r1, (short)(r2 >> 16), ((short)r2 >> 4) & 0x0FFF, ((short)(r3 >> 16) & 0x3FFF) | 0x8000, (short)r3, r4);
-#else
-  sprintf(uuid, "urn:uuid:%8.8x-%4.4hx-4%3.3hx-%4.4hx-%4.4hx%8.8x", r1, (short)(r2 >> 16), ((short)r2 >> 4) & 0x0FFF, ((short)(r3 >> 16) & 0x3FFF) | 0x8000, (short)r3, r4);
-#endif
+  (SOAP_SNPRINTF(uuid, uuidlen, 45), "urn:uuid:%8.8x-%4.4hx-4%3.3hx-%4.4hx-%4.4hx%8.8x", r1, (short)(r2 >> 16), (short)(((short)r2 >> 4) & 0x0FFF), (short)(((short)(r3 >> 16) & 0x3FFF) | 0x8000), (short)r3, r4);
   DBGFUN1("soap_wsa_rand_uuid", "%s", uuid);
   return uuid;
 }
@@ -1363,7 +1359,8 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultSubcodeValues) fault)
 */
 int
 soap_wsa(struct soap *soap, struct soap_plugin *p, void *arg)
-{ DBGFUN("soap_wsa");
+{ (void)arg;
+  DBGFUN("soap_wsa");
   p->id = soap_wsa_id;
   p->data = (void*)SOAP_MALLOC(soap, sizeof(struct soap_wsa_data));
   p->fcopy = NULL;
