@@ -49,7 +49,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 */
 
 #ifndef WSDDAPI_H
-#define WSSDDPI_H
+#define WSDDAPI_H
 
 #include "wsaapi.h"	/* also includes soapH.h, change wsaapi.h if needed */
 #include "threads.h"	/* threads and locks from plugin/threads.h */
