@@ -40,9 +40,9 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 int main()
 {
   mashupProxy proxy;
-  _ns5__commingtotown response;
+  _ns3__commingtotown response;
 
-  if (proxy.dtx((char*)"", &response))
+  if (proxy.dtx((char*)"", response))
     proxy.soap_stream_fault(std::cerr);
   else if (response.days == 0)
     std::cout << "Today is the day!" << std::endl;
