@@ -39,7 +39,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "soapmashupService.h"
 #include "mashup.nsmap"
 
-int main(int argc, char **argv)
+int main()
 {
   mashupService service;
 
@@ -54,6 +54,8 @@ int main(int argc, char **argv)
 
 int mashupService::dtx(_XML x, _ns3__commingtotown &response)
 {
+  (void)x; /* input param that is always empty is simply ignored */
+
   ServiceProxy Time("http://www.cs.fsu.edu/~engelen/gmtlitserver.cgi");
 
   _ns1__gmt gmt;
