@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2016, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -37,7 +37,11 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 #include "stdsoap2.h"
 
-#define WSDL2H_VERSION "2.8.16"
+#ifdef WITH_OPENSSL
+#include "httpda.h"
+#endif
+
+#define WSDL2H_VERSION "2.8.28"
 
 #ifdef WIN32
 # pragma warning(disable : 4996)
@@ -90,6 +94,7 @@ extern int _flag,
            aflag,
            bflag,
 	   cflag,
+	   c11flag,
 	   dflag,
 	   eflag,
 	   fflag,
@@ -102,6 +107,7 @@ extern int _flag,
 	   Pflag,
 	   Rflag,
 	   sflag,
+	   Uflag,
 	   uflag,
 	   vflag,
 	   wflag,
@@ -116,8 +122,9 @@ extern SetOfString exturis;
 
 #define MAXINFILES (1000)
 
+extern int openfiles;
 extern int infiles;
-extern char *infile[MAXINFILES], *outfile, *proxy_host, *proxy_userid, *proxy_passwd;
+extern char *infile[MAXINFILES], *outfile, *proxy_host, *proxy_userid, *proxy_passwd, *auth_userid, *auth_passwd;
 extern const char *mapfile, *import_path, *cwd_path, *cppnamespace;
 
 extern int proxy_port;
@@ -128,9 +135,10 @@ extern const char *schema_prefix;
 extern const char elementformat[];
 extern const char pointerformat[];
 extern const char attributeformat[];
-extern const char vectorformat_open[];
-extern const char vectorformat[];
-extern const char pointervectorformat[];
+extern const char templateformat_open[];
+extern const char attrtemplateformat_open[];
+extern const char templateformat[];
+extern const char pointertemplateformat[];
 extern const char arrayformat[];
 extern const char arraysizeformat[];
 extern const char arrayoffsetformat[];
