@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2014, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -40,13 +40,13 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 
 class Message
 { public:
+    wsdl__message *message;
     const char *name;
     const char *URI;
     soap__styleChoice style;
     soap__useChoice use;
     const char *encodingStyle;
     const char *action;
-    wsdl__message *message;
     xs__element *element;
     const char *body_parts;
     wsdl__part *part;
@@ -66,6 +66,7 @@ typedef map<const char*, Message*, ltstr> MapOfStringToMessage;
 
 class Operation
 { public:
+    const wsdl__operation *operation;
     const char *prefix;
     const char *URI;
     const char *name;
@@ -101,7 +102,8 @@ class Service
     MapOfStringToString service_documentation;
     MapOfStringToString port_documentation;
     MapOfStringToString binding_documentation;
-    vector<const wsp__Policy*> policy;
+    vector<const wsp__Policy*> policy;	// WS-Policy
+    vector<const plnk__tRole*> role;	// BPEL 2.0
     VectorOfString imports;
     Service();
     void generate(Types&);
@@ -116,6 +118,7 @@ class Definitions
     Types types;				// to process schema type information
     MapOfStringToService services;		// service information gathered
     bool soap12;
+    int binding_count;
     Definitions();
     void collect(const wsdl__definitions&);
     void compile(const wsdl__definitions&);
