@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2000-2013, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2016, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -60,6 +60,7 @@ int _flag = 0,
     aflag = 0,
     bflag = 0,
     cflag = 0,
+    c11flag = 0,
     dflag = 0,
     eflag = 0,
     fflag = 0,
@@ -73,6 +74,7 @@ int _flag = 0,
     Rflag = 0,
     sflag = 0,
     uflag = 0,
+    Uflag = 0,
     vflag = 0,
     wflag = 0,
     Wflag = 0,
@@ -80,6 +82,7 @@ int _flag = 0,
     yflag = 0,
     zflag = 0;
 
+int openfiles = 0;
 int infiles = 0;
 char *infile[MAXINFILES],
      *outfile = NULL,
@@ -108,9 +111,10 @@ const char *schema_prefix = "ns";
 const char elementformat[]             = "    %-35s  %-30s";
 const char pointerformat[]             = "    %-35s *%-30s";
 const char attributeformat[]           = "   @%-35s  %-30s";
-const char vectorformat[]              = "    std::vector<%-23s> %-30s";
-const char pointervectorformat[]       = "    std::vector<%-22s> *%-30s";
-const char vectorformat_open[]         = "    std::vector<%s";
+const char templateformat[]            = "    %s<%-23s> %-30s";
+const char pointertemplateformat[]     = "    %s<%-22s> *%-30s";
+const char templateformat_open[]       = "    %s<%s";
+const char attrtemplateformat_open[]   = "   @%s<%s";
 const char arrayformat[]               = "    %-35s *__ptr%-25s";
 const char arraysizeformat[]           = "    %-35s  __size%-24s";
 const char arrayoffsetformat[]         = "//  %-35s  __offset%-22s";
@@ -121,15 +125,15 @@ const char serviceformat[]             = "//gsoap %-4s service %s:\t%s %s\n";
 const char paraformat[]                = "    %-35s%s%s%s";
 const char anonformat[]                = "    %-35s%s_%s%s";
 
-const char copyrightnotice[] = "\n**  The gSOAP WSDL/Schema processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2013 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool is released under one of the following licenses:\n**  GPL or the commercial license by Genivia Inc. Use option -l for details.\n\n";
+const char copyrightnotice[] = "\n**  The gSOAP WSDL/Schema processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2016 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool and its generated software are released under the GPL.\n**  ----------------------------------------------------------------------------\n**  A commercial use license is available from Genivia Inc., contact@genivia.com\n**  ----------------------------------------------------------------------------\n\n";
 
 const char licensenotice[]   = "\
 --------------------------------------------------------------------------------\n\
 gSOAP XML Web services tools\n\
-Copyright (C) 2000-2013, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
-\n\
-This software is released under one of the following licenses:\n\
-GPL or Genivia's license for commercial use.\n\
+Copyright (C) 2000-2016, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
+The wsdl2h tool and its generated software are released under the GPL.\n\
+This software is released under the GPL with the additional exemption that\n\
+compiling, linking, and/or using OpenSSL is allowed.\n\
 --------------------------------------------------------------------------------\n\
 GPL license.\n\
 \n\
@@ -156,23 +160,28 @@ A commercial-use license is available from Genivia, Inc., contact@genivia.com\n\
 --------------------------------------------------------------------------------\n";
 
 int main(int argc, char **argv)
-{ init();
+{
   fprintf(stderr, "%s", copyrightnotice);
   options(argc, argv);
+  init();
+  if (cppnamespace)
+    fprintf(stream, "namespace %s {\n", cppnamespace);
   Definitions def;
   wsdl__definitions definitions;
   definitions.read(infiles, infile);
   if (definitions.error())
-  { definitions.print_fault();
+  {
+    definitions.print_fault();
     exit(1);
   }
   definitions.traverse();
   def.compile(definitions);
   if (outfile)
-  { fclose(stream);
-    fprintf(stderr, "\nTo complete the process, compile with:\n> soapcpp2 %s\n", outfile);
+  {
+    fclose(stream);
+    fprintf(stderr, "\nTo finalize code generation, execute:\n> soapcpp2 %s\n", outfile);
     if (!cflag)
-      fprintf(stderr, "or to generate C++ proxy and object classes:\n> soapcpp2 -j %s\n", outfile);
+      fprintf(stderr, "Or to generate C++ proxy and service classes:\n> soapcpp2 -j %s\n", outfile);
     fprintf(stderr, "\n");
   }
   return 0;
@@ -185,10 +194,13 @@ int main(int argc, char **argv)
 ////////////////////////////////////////////////////////////////////////////////
 
 static void init()
-{ struct Namespace *p = namespaces;
+{
+  struct Namespace *p = namespaces;
   if (p)
-  { for (; p->id; p++)
-    { if (p->in && *p->in)
+  {
+    for (; p->id; p++)
+    {
+      if (p->in && *p->in)
         exturis.insert(p->in);
       if (p->ns && *p->ns)
         exturis.insert(p->ns);
@@ -203,19 +215,25 @@ static void init()
 ////////////////////////////////////////////////////////////////////////////////
 
 static void options(int argc, char **argv)
-{ int i;
+{
+  int i;
   infiles = 0;
+  openfiles = 0;
   for (i = 1; i < argc; i++)
-  { char *a = argv[i];
+  {
+    char *a = argv[i];
     if (*a == '-'
 #ifdef WIN32
      || *a == '/'
 #endif
     )
-    { int g = 1;
+    {
+      int g = 1;
       while (g && *++a)
-      { switch (*a)
-        { case '_':
+      {
+	switch (*a)
+        {
+	  case '_':
             _flag = 1;
        	    break;
           case 'a':
@@ -225,9 +243,22 @@ static void options(int argc, char **argv)
             bflag = 1;
        	    break;
           case 'c':
-            cflag = 1;
-            if (cppnamespace)
-	      fprintf(stderr, "wsdl2h: Options -c and -q clash\n");
+	    if (a[1] == '+' && a[2] == '+')
+	    {
+	      a += 2;
+	      if (a[1] == '1' && a[2] == '1')
+	      {
+		a += 2;
+		c11flag = 1;
+	      }
+	      cflag = 0;
+	    }
+	    else
+	    {
+	      cflag = 1;
+	      if (cppnamespace)
+		fprintf(stderr, "wsdl2h: Options -c and -q cannot be used together\n");
+	    }
        	    break;
 	  case 'd':
 	    dflag = 1;
@@ -313,7 +344,7 @@ static void options(int argc, char **argv)
             else
               fprintf(stderr, "wsdl2h: Option -q requires a C++ namespace name argument\n");
             if (cflag)
-	      fprintf(stderr, "wsdl2h: Options -c and -q clash\n");
+	      fprintf(stderr, "wsdl2h: Options -c and -q cannot be used together\n");
 	    break;
 	  case 'r':
             a++;
@@ -325,31 +356,39 @@ static void options(int argc, char **argv)
             else
               fprintf(stderr, "wsdl2h: Option -r requires proxy host:port:userid:passwd or :userid:passwd authentication argument\n");
             if (proxy_host)
-	    { char *s = (char*)emalloc(strlen(proxy_host + 1));
-	      strcpy(s, proxy_host);
+	    {
+	      size_t l = strlen(proxy_host);
+	      char *s = (char*)emalloc(l + 1);
+	      soap_strcpy(s, l + 2, proxy_host);
 	      proxy_host = s;
 	      s = strchr(proxy_host, ':');
 	      if (s)
-	      { *s = '\0';
+	      {
+		*s = '\0';
 		if (*proxy_host)
-	        { proxy_port = soap_strtol(s + 1, &s, 10);
+	        {
+		  proxy_port = soap_strtol(s + 1, &s, 10);
 		  if (s && *s == ':')
-	          { *s = '\0';
+	          {
+		    *s = '\0';
 		    proxy_userid = s + 1;
 		    s = strchr(proxy_userid, ':');
 		    if (s && *s == ':')
-		    { *s = '\0';
+		    {
+		      *s = '\0';
 		      proxy_passwd = s + 1;
 		    }
 		  }
 	        }
 		else
-		{ s = proxy_host;
+		{
+		  s = proxy_host;
 		  proxy_host = NULL;
 		  auth_userid = s + 1;
                   s = strchr(auth_userid, ':');
 		  if (s && *s == ':')
-		  { *s = '\0';
+		  {
+		    *s = '\0';
 		    auth_passwd = s + 1;
 		  }
 		}
@@ -372,6 +411,9 @@ static void options(int argc, char **argv)
             else
               fprintf(stderr, "wsdl2h: Option -t requires a type map file argument\n");
 	    break;
+	  case 'U':
+	    Uflag = 1;
+	    break;
 	  case 'u':
 	    uflag = 1;
 	    break;
@@ -402,11 +444,13 @@ static void options(int argc, char **argv)
 	    break;
           case '?':
           case 'h':
-            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c] [-d] [-e] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-l] [-m] [-N name] [-n name] [-P|-p] [-q name] [-R] [-r proxyhost[:port[:uid:pwd]]] [-r:userid:passwd] [-s] [-t typemapfile] [-u] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
+            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c|-c++|-c++11] [-d] [-e] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-l] [-m] [-N name] [-n name] [-P|-p] [-q name] [-R] [-r proxyhost[:port[:uid:pwd]]] [-r:userid:passwd] [-s] [-t typemapfile] [-U] [-u] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
             fprintf(stderr, "\
 -a      generate indexed struct names for local elements with anonymous types\n\
 -b	bi-directional operations (duplex ops) added to serve one-way responses\n\
 -c      generate C source code\n\
+-c++    generate C++ source code (default)\n\
+-c++11  generate C++11 source code\n\
 -d      use DOM to populate xs:any, xs:anyType, and xs:anyAttribute\n\
 -e      don't qualify enum names\n\
 -f      generate flat C++ class hierarchy\n\
@@ -431,6 +475,7 @@ static void options(int argc, char **argv)
         connect with authentication credentials (digest auth requires SSL)\n\
 -s      don't generate STL code (no std::string and no std::vector)\n\
 -tfile  use type map file instead of the default file typemap.dat\n\
+-U      allow UTF8-encoded Unicode C/C++ identifiers when mapping XML tag names\n\
 -u      don't generate unions\n\
 -v      verbose output\n\
 -W      suppress warnings\n\
@@ -442,6 +487,7 @@ static void options(int argc, char **argv)
 -z3     compatibility with 2.7.16 to 2.8.7: qualify element/attribute references\n\
 -z4     compatibility up to 2.8.11: don't generate union structs in std::vector\n\
 -z5     compatibility up to 2.8.15\n\
+-z6     compatibility up to 2.8.17\n\
 -_      don't generate _USCORE (replace with UNICODE _x005f)\n\
 infile.wsdl infile.xsd http://www... list of input sources (if none: use stdin)\n\
 \n");
@@ -453,37 +499,45 @@ infile.wsdl infile.xsd http://www... list of input sources (if none: use stdin)\
       }
     }
     else
-    { infile[infiles++] = argv[i];
+    {
+      infile[infiles++] = argv[i];
       if (infiles >= MAXINFILES)
-      { fprintf(stderr, "wsdl2h: too many files\n");
+      {
+	fprintf(stderr, "wsdl2h: too many files\n");
         exit(1);
       }
     }
   }
   if (infiles)
-  { if (!outfile)
-    { if (strncmp(infile[0], "http://", 7) && strncmp(infile[0], "https://", 8))
-      { const char *s = strrchr(infile[0], '.');
+  {
+    if (!outfile)
+    {
+      if (strncmp(infile[0], "http://", 7) && strncmp(infile[0], "https://", 8))
+      {
+	const char *s = strrchr(infile[0], '.');
         if (s && (!soap_tag_cmp(s, ".wsdl") || !soap_tag_cmp(s, ".gwsdl") || !soap_tag_cmp(s, ".xsd")))
-        { outfile = estrdup(infile[0]);
+        {
+	  outfile = estrdup(infile[0]);
           outfile[s - infile[0] + 1] = 'h';
           outfile[s - infile[0] + 2] = '\0';
         }
         else
-        { outfile = (char*)emalloc(strlen(infile[0]) + 3);
-          strcpy(outfile, infile[0]);
-          strcat(outfile, ".h");
+        {
+	  size_t l = strlen(infile[0]);
+	  outfile = (char*)emalloc(l + 3);
+          soap_strcpy(outfile, l + 3, infile[0]);
+          soap_strcpy(outfile + l, 3, ".h");
         }
       }
     }
     if (outfile)
-    { stream = fopen(outfile, "w");
+    {
+      stream = fopen(outfile, "w");
       if (!stream)
-      { fprintf(stderr, "Cannot write to %s\n", outfile);
+      {
+	fprintf(stderr, "Cannot write to %s\n", outfile);
         exit(1);
       }
-      if (cppnamespace)
-        fprintf(stream, "namespace %s {\n", cppnamespace);
       fprintf(stderr, "Saving %s\n\n", outfile);
     }
   }
@@ -503,25 +557,32 @@ struct Namespace namespaces[] =
   {"xsd", "-"}, // http://www.w3.org/2001/XMLSchema"}, // don't use this, it might conflict with xs
   {"xml", "http://www.w3.org/XML/1998/namespace"},
   {"xs", "http://www.w3.org/2001/XMLSchema", "http://www.w3.org/*/XMLSchema" },
+  {"vc", "http://www.w3.org/2007/XMLSchema-versioning" },
   {"http", "http://schemas.xmlsoap.org/wsdl/http/"},
   {"soap", "http://schemas.xmlsoap.org/wsdl/soap/", "http://schemas.xmlsoap.org/wsdl/soap*/"},
   {"mime", "http://schemas.xmlsoap.org/wsdl/mime/"},
   {"xmime", "http://www.w3.org/2005/05/xmlmime"},
   {"dime", "http://schemas.xmlsoap.org/ws/2002/04/dime/wsdl/", "http://schemas.xmlsoap.org/ws/*/dime/wsdl/"},
-  {"sp", "http://docs.oasis-open.org/ws-sx/ws-securitypolicy/200702", "http://schemas.xmlsoap.org/ws/2005/07/securitypolicy"},
   {"wsdl", "http://schemas.xmlsoap.org/wsdl/", "http://www.w3.org/ns/wsdl"},
   {"wsdli", "http://www.w3.org/ns/wsdl-instance"},
   {"wsdlx", "http://www.w3.org/ns/wsdl-extensions"},
   {"wsoap", "http://www.w3.org/ns/wsdl/soap"},
   {"whttp", "http://www.w3.org/ns/wsdl/http"},
   {"wrpc", "http://www.w3.org/ns/wsdl/rpc"},
-  {"wsa_", "http://www.w3.org/2005/08/addressing"},
   {"wsaw", "http://www.w3.org/2006/05/addressing/wsdl"},
+  {"gwsdl", "http://www.gridforum.org/namespaces/2003/03/gridWSDLExtensions"},
+  {"wsa_", "http://www.w3.org/2005/08/addressing"},
   {"wsam", "http://www.w3.org/2007/05/addressing/metadata"},
   {"wsrmp", "http://schemas.xmlsoap.org/ws/2005/02/rm/policy", "http://docs.oasis-open.org/ws-rx/wsrmp/*"},
-  {"wsp", "http://www.w3.org/ns/ws-policy", "http://schemas.xmlsoap.org/ws/2004/09/policy"},
-  {"wst", "http://docs.oasis-open.org/ws-sx/ws-trust/200512"},
+  {"sp", "http://docs.oasis-open.org/ws-sx/ws-securitypolicy/200702", "http://schemas.xmlsoap.org/ws/2005/07/securitypolicy"},
+  {"wsp_", "http://www.w3.org/ns/ws-policy", "http://schemas.xmlsoap.org/ws/2004/09/policy"},
+  {"wst_", "http://docs.oasis-open.org/ws-sx/ws-trust/200512"},
   {"wsu_", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"},
-  {"gwsdl", "http://www.gridforum.org/namespaces/2003/03/gridWSDLExtensions"},
+  {"plnk","http://docs.oasis-open.org/wsbpel/2.0/plnktype"},
+  {"vprop","http://docs.oasis-open.org/wsbpel/2.0/varprop"},
+#ifdef WITH_BPEL
+  {"bpel","http://docs.oasis-open.org/wsbpel/2.0/process/executable"},
+  {"sref","http://docs.oasis-open.org/wsbpel/2.0/serviceref"},
+#endif
   {NULL, NULL}
 };
